class Node:
    def __init__(self, data):
        self.data = data
        self.next = None


class LinkedListIterator:
    def __init__(self, head, reversed=False):
        self.current = head
        if reversed:
            self.nodes = []
            while head:
                self.nodes.append(head)
                head = head.next
            self.nodes.reverse()
            self.index = 0

    def __iter__(self):
        return self

    def __next__(self):
        if hasattr(self, "nodes"):
            if self.index < len(self.nodes):
                data = self.nodes[self.index].data
                self.index += 1
                return data
            else:
                raise StopIteration
        else:
            if not self.current:
                raise StopIteration
            data = self.current.data
            self.current = self.current.next
            return data


class LinkedList:
    def __init__(self):
        self.head = None

    def append(self, data):
        new_node = Node(data)
        if not self.head:
            self.head = new_node
        else:
            current = self.head
            while current.next:
                current = current.next
            current.next = new_node

    def __len__(self):
        count = 0
        current = self.head
        while current:
            count += 1
            current = current.next
        return count

    def __add__(self, other):
        if not isinstance(other, LinkedList):
            return NotImplemented
        new_list = LinkedList()
        current = self.head
        while current:
            new_list.append(current.data)
            current = current.next
        current = other.head
        while current:
            new_list.append(current.data)
            current = current.next
        return new_list

    def __mul__(self, n):
        if not isinstance(n, int):
            return NotImplemented
        if n <= 0:
            return LinkedList()
        new_list = LinkedList()
        for _ in range(n):
            current = self.head
            while current:
                new_list.append(current.data)
                current = current.next
        return new_list

    def __rmul__(self, n):
        return self.__mul__(n)

    def __iadd__(self, other):
        if not isinstance(other, LinkedList):
            return NotImplemented
        current = self.head
        if not current:
            self.head = other.head
        else:
            last = current
            while last.next:
                last = last.next
            last.next = other.head
        return self

    def __iter__(self):
        return LinkedListIterator(self.head)

    def __reversed__(self):
        return LinkedListIterator(self.head, reversed=True)

    def __imul__(self, n):
        if not isinstance(n, int):
            return NotImplemented
        original_data = list(self)
        if n <= 0:
            self.head = None
        else:
            self.head = None
            for _ in range(n):
                for item in original_data:
                    self.append(item)
        return self

    def __contains__(self, item):
        current = self.head
        while current:
            if current.data == item:
                return True
            current = current.next
        return False

    def __getitem__(self, index):
        if index < 0:
            raise IndexError("Negative indices are not supported")
        current = self.head
        for _ in range(index):
            if current is None:
                raise IndexError("Index out of bounds")
            current = current.next
        if current is None:
            raise IndexError("Index out of bounds")
        return current.data

    def __setitem__(self, index, value):
        if index < 0:
            raise IndexError("Negative indices are not supported")
        current = self.head
        for _ in range(index):
            if current is None:
                raise IndexError("Index out of bounds")
            current = current.next
        if current is None:
            raise IndexError("Index out of bounds")
        current.data = value

    def __delitem__(self, index):
        if index < 0:
            raise IndexError("Negative indices are not supported")
        if self.head is None:
            raise IndexError("Index out of bounds")
        if index == 0:
            self.head = self.head.next
            return
        current = self.head
        previous = None
        for _ in range(index):
            previous = current
            if current is None or current.next is None:
                raise IndexError("Index out of bounds")
            current = current.next
        if current is None:
            raise IndexError("Index out of bounds")
        previous.next = current.next

    def __eq__(self, other):
        if not isinstance(other, LinkedList):
            return NotImplemented
        return list(self) == list(other)

    def __ne__(self, other):
        return not self.__eq__(other)

    def __lt__(self, other):
        if not isinstance(other, LinkedList):
            return NotImplemented
        return list(self) < list(other)

    def __le__(self, other):
        return self.__lt__(other) or self.__eq__(other)

    def __gt__(self, other):
        if not isinstance(other, LinkedList):
            return NotImplemented
        return list(self) > list(other)

    def __ge__(self, other):
        return self.__gt__(other) or self.__eq__(other)

    def display(self):
        current = self.head
        while current:
            print(current.data, end=" ")
            current = current.next
        print()


def main():
    # 创建第一个 LinkedList 实例，并添加草丛三姐妹
    heroes_list = LinkedList()
    heroes_list.append("妲己")
    heroes_list.append("安琪拉")
    heroes_list.append("王昭君")

    # 创建第二个 LinkedList 实例，并添加其他英雄
    more_heroes_list = LinkedList()
    more_heroes_list.append("露娜")
    more_heroes_list.append("娜可露露")
    more_heroes_list.append("貂蝉")

    # 使用比较操作测试
    print("\nTesting comparison methods:")
    print("Is the first list equal to the second list?:", heroes_list == more_heroes_list)
    print("Is the first list not equal to the second list?:", heroes_list != more_heroes_list)
    print("Is the first list less than the second list?:", heroes_list < more_heroes_list)
    print("Is the first list less than or equal to the second list?:", heroes_list <= more_heroes_list)
    print("Is the first list greater than the second list?:", heroes_list > more_heroes_list)
    print("Is the first list greater than or equal to the second list?:", heroes_list >= more_heroes_list)

    # 显示第一个链表的英雄名字和长度
    print("First LinkedList:")
    heroes_list.display()
    print("Number of heroes in the first linked list:", len(heroes_list))

    # 显示第二个链表的英雄名字和长度
    print("\nSecond LinkedList:")
    more_heroes_list.display()
    print("Number of heroes in the second linked list:", len(more_heroes_list))

    # 使用 __add__ 方法合并两个链表
    combined_list = heroes_list + more_heroes_list

    # 显示合并后的链表的英雄名字和长度
    print("\nCombined LinkedList:")
    combined_list.display()
    print("Number of heroes in the combined linked list:", len(combined_list))

    # 使用 __mul__ 方法重复链表内容
    tripled_list = heroes_list * 3
    print("\nTripled Heroes List using __mul__:")
    tripled_list.display()

    # 使用 __imul__ 方法就地重复链表内容
    heroes_list *= 3
    print("\nHeroes List after using __imul__ (tripled in-place):")
    heroes_list.display()
    print("\nNumber of heroes after __imul__:", len(heroes_list))

    # 使用 for 循环迭代 LinkedList
    print("\nIterating over the second list:")
    for data in more_heroes_list:
        print(data, end=" ")

    print()

    # 使用 __contains__ 方法实现检查某个元素是否在LinkedList对象中
    print("\nContains method test:")
    print("'安琪拉' is in heroes_list:", "安琪拉" in heroes_list)
    print("'安琪拉' is in more_heroes_list:", "安琪拉" in more_heroes_list)

    # 使用 __getitem__ 访问元素
    print("\nAccessing first element of the second list:", more_heroes_list[0])

    # 使用 __setitem__ 修改元素
    more_heroes_list[2] = "孙悟空"
    print("\nHeros List after using __setitem__: ")
    more_heroes_list.display()

    # 使用 __delitem__ 删除元素
    del more_heroes_list[2]
    print("\nHeros List after using __delitem__: ")
    more_heroes_list.display()

if __name__ == "__main__":
    main()
